% Generate an adjacency matrix, the indices of original data in adj, and
% the indices to return from adj ordering to original ordering
% NOTE: code for adjacency matrix is taken from function monreggav.m of:
%     Authors: Burdakov O., Grimvall A., Sysoev O.
%     Reference: O. Burdakov, O. Sysoev, A. Grimvall and M. Hussian. 
%                "An O(n2) algorithm for isotonic regression."
%                 In: G. Di Pillo and M. Roma (Eds) Large-Scale Nonlinear Optimization.
%                 Series: Nonconvex Optimization and Its Applications,
%                 Springer-Verlag, (2006) 83, pp. 25-33
%     Available: http://www.mai.liu.se/~olbur/SOFTWARE/GPAV/
%%%%%%%%%%%%
% Inputs:  
% data:           m x n data matrix, m samples and n features
%%%%%%%%%%%
% Outputs:  
% adj:            adjacency matrix with adj(i,j)==1 if y(i)<=y(j) and adj(i,j)==0 otherwise
% ind_new:        indices of data entries that are used in adj, i.e. adj(i,j) compares data(ind_new(i),:) and data(ind_new(j),:)
% ind_original:   indices of original order of data, i.e. adj(ind_original(i),ind_original(j)) compares data(i,:) and data(j,:)
%%%%%%%%%%%
% Modified by Ronny Luss, May 2011
%%%%%%%%%%%

function [adj,ind_new,ind_original] = adjacency_matrix(data)

[N,p] = size(data);

[data,order] = sortrows(data);
sortX = data(:,1:p);

adj=logical(sparse(N,N));

sortX=sortX';

degree=uint16(zeros(N,1));

for i=2:N
    pred = false(1,N);
    pred(1:i-1)=min(sortX(:,1:(i-1))<=repmat(sortX(:,i),1,i-1));
    degree(i)=sum(pred);
    p=find(pred);
    [pp,temp]=find(adj(:,p));
    pred(pp)=0;
    adj(:,i)=pred';
end;

[temp,degree]=sort(degree);
adj=adj(degree,degree);

degreem(degree)=1:N;
ordm(order)=1:N;
ind_original=degreem(ordm);
[temp,ind_new]=sort(ind_original,'ascend');
